@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2012  Jendrik Seipp
 #
@@ -24,6 +23,7 @@ from gi.repository import Gtk
 from rednotebook.gui import customwidgets
 from rednotebook.util import filesystem
 from rednotebook.util import dates
+from rednotebook.util import urls
 
 
 MENUITEMS_XML = '''\
@@ -64,7 +64,7 @@ class InsertMenu:
         self.main_window = main_window
 
         self.bullet_list = (
-            '\n- %s\n- %s\n  - %s (%s)\n\n\n' % (
+            '\n- {}\n- {}\n  - {} ({})\n\n\n'.format(
                 _('First Item'), _('Second Item'), _('Indented Item'),
                 _('Two blank lines close the list')))
 
@@ -230,9 +230,9 @@ class InsertMenu:
 
                 # On windows firefox accepts absolute filenames only
                 # with the file:// prefix
-                base = filesystem.get_local_url(base)
+                base = urls.get_local_url(base)
 
-                lines.append('[%s""%s""%s%s]' % (sel_text, base, ext, width_text))
+                lines.append('[{}""{}""{}{}]'.format(sel_text, base, ext, width_text))
 
             return '\n'.join(lines)
 
@@ -250,11 +250,11 @@ class InsertMenu:
             if folder:
                 dirs.last_file_dir = folder
             filename = file_chooser.get_filename()
-            filename = filesystem.get_local_url(filename)
+            filename = urls.get_local_url(filename)
             sel_text = self.main_window.day_text_field.get_selected_text()
             _, tail = os.path.split(filename)
             # It is always safer to add the "file://" protocol and the ""s
-            return '[%s ""%s""]' % (sel_text or tail, filename)
+            return '[{} ""{}""]'.format(sel_text or tail, filename)
 
     def on_insert_link(self, sel_text):
         link_creator = self.main_window.builder.get_object('link_creator')
@@ -292,7 +292,7 @@ class InsertMenu:
             link_name = link_name_entry.get_text()
 
             if link_location and link_name:
-                return '[%s ""%s""]' % (link_name, link_location)
+                return '[{} ""{}""]'.format(link_name, link_location)
             elif link_location:
                 return link_location
             else:
@@ -316,5 +316,5 @@ class InsertMenu:
     #     return '\\(', formula, '\\)'
 
     def on_insert_date_time(self, sel_text):
-        format_string = self.main_window.journal.config.read('dateTimeString', '%A, %x %X')
+        format_string = self.main_window.journal.config.read('dateTimeString')
         return dates.format_date(format_string)
