@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009  Jendrik Seipp
 #
@@ -17,15 +16,18 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
+from collections import OrderedDict
 import datetime
 import logging
 import os
+import urllib.parse
 from unittest import mock
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import GObject
 from gi.repository import Gtk
+from gi.repository import GtkSource
 from gi.repository import Pango
 
 from rednotebook.gui.menu import MainMenuBar
@@ -37,11 +39,12 @@ from rednotebook import info
 from rednotebook import templates
 from rednotebook.util import dates
 from rednotebook.util import markup
+from rednotebook.util import urls
 from rednotebook.util import utils
-from rednotebook import undo
 from rednotebook.gui import categories
 from rednotebook.gui.exports import ExportAssistant
 from rednotebook.gui import browser
+from rednotebook.gui import browser_cef
 from rednotebook.gui import search
 from rednotebook.gui import editor
 from rednotebook.gui import insert_menu
@@ -61,6 +64,9 @@ class MainWindow:
         # TODO: Remove workaround for Windows once it is no longer needed.
         self.gladefile = os.path.join(filesystem.files_dir, 'main_window.glade')
         self.builder = Gtk.Builder()
+        # Register GtkSourceView so builder can use it when loading the file
+        # https://stackoverflow.com/q/10524196/434217
+        GObject.type_register(GtkSource.View)
         if filesystem.IS_WIN:
             import xml.etree.ElementTree as ET
             tree = ET.parse(self.gladefile)
@@ -94,11 +100,13 @@ class MainWindow:
         main_vbox.pack_start(self.menubar, False, False, 0)
         main_vbox.reorder_child(self.menubar, 0)
 
-        self.undo_redo_manager = undo.UndoRedoManager(self)
+        self.undo_action = self.uimanager.get_action('/MainMenuBar/Edit/Undo')
+        self.redo_action = self.uimanager.get_action('/MainMenuBar/Edit/Redo')
 
         self.calendar = MainCalendar(self.journal, self.builder.get_object('calendar'))
-        self.day_text_field = DayEditor(self.builder.get_object('day_text_view'),
-                                        self.undo_redo_manager)
+        self.day_text_field = DayEditor(self.builder.get_object('day_text_view'))
+        self.day_text_field.connect('can-undo-redo-changed', self.update_undo_redo_buttons)
+        self.update_undo_redo_buttons()
         self.day_text_field.day_text_view.grab_focus()
         can_spell_check = self.day_text_field.can_spell_check()
         spell_check_enabled = bool(self.journal.config.read('spellcheck'))
@@ -126,28 +134,50 @@ class MainWindow:
         self.edit_pane = self.builder.get_object('edit_pane')
         self.text_vbox = self.builder.get_object('text_vbox')
 
+        use_cef = True
         if browser.WebKit2:
             class Preview(browser.HtmlView):
                 def __init__(self, journal):
                     browser.HtmlView.__init__(self)
                     self.journal = journal
+                    self.internal = True
 
                 def show_day(self, new_day):
                     html = self.journal.convert(new_day.text, 'xhtml')
                     self.load_html(html)
 
+                def shutdown(self):
+                    pass
+
             self.html_editor = Preview(self.journal)
             self.html_editor.connect('button-press-event', self.on_browser_clicked)
             self.html_editor.connect('decide-policy', self.on_browser_decide_policy)
-
             self.text_vbox.pack_start(self.html_editor, True, True, 0)
-            self.html_editor.hide()
             self.html_editor.set_editable(False)
+        elif browser_cef.cef and use_cef:
+            class Preview(browser_cef.HtmlView):
+                def __init__(self, journal):
+                    super().__init__()
+                    self.journal = journal
+                    self.internal = True
+
+                def show_day(self, new_day):
+                    html = self.journal.convert(new_day.text, 'xhtml')
+                    self.load_html(html)
+
+                def highlight(self, text):
+                    pass
+
+            self.html_editor = Preview(self.journal)
+            self.text_vbox.pack_start(self.html_editor, True, True, 0)
         else:
             self.html_editor = mock.MagicMock()
+            self.html_editor.internal = False
             preview_button = self.builder.get_object('preview_button')
             preview_button.set_label(_('Preview in Browser'))
 
+        self.html_editor.hide()
+
         self.preview_mode = False
 
         # Let the edit_paned respect its childs size requests
@@ -273,7 +303,7 @@ class MainWindow:
         else:
             self.show()
 
-    def on_tray_popup_menu(self, status_icon, button, activate_time):
+    def on_tray_popup_menu(self, _status_icon, button, activate_time):
         '''
         Called when the user right-clicks the tray icon
         '''
@@ -305,9 +335,7 @@ class MainWindow:
         # Create a Menu
         menu = self.uimanager.get_widget('/TrayMenu')
 
-        menu.popup(
-            None, None, Gtk.status_icon_position_menu, button,
-            activate_time, status_icon)
+        menu.popup(None, None, None, None, button, activate_time)
 
     def show(self):
         self.main_frame.show()
@@ -327,6 +355,7 @@ class MainWindow:
         if self.journal.config.read('closeToTray'):
             self.hide()
         else:
+            self.html_editor.shutdown()
             self.journal.exit()
 
         # We never call the default handler. Otherwise, the window would be
@@ -351,8 +380,8 @@ class MainWindow:
         day_list = CustomListView(columns)
         overall_box.pack_start(overall_list, True, True, 0)
         day_box.pack_start(day_list, True, True, 0)
-        setattr(self.stats_dialog, 'overall_list', overall_list)
-        setattr(self.stats_dialog, 'day_list', day_list)
+        self.stats_dialog.overall_list = overall_list
+        self.stats_dialog.day_list = day_list
         for list in [overall_list, day_list]:
             list.set_headers_visible(False)
 
@@ -375,7 +404,7 @@ class MainWindow:
             edit_button.show()
             preview_button.hide()
 
-            self.undo_redo_manager.disable_buttons()
+            self.disable_undo_redo_buttons()
         else:
             # Enter edit mode
             edit_scroll.show()
@@ -384,7 +413,11 @@ class MainWindow:
             preview_button.show()
             edit_button.hide()
 
-            self.undo_redo_manager.update_buttons()
+            self.update_undo_redo_buttons()
+
+        # Interacting with the CEF browser makes the main window inactive, so
+        # we make it active again.
+        self.main_frame.present()
 
         self.template_manager.set_template_menu_sensitive(not preview)
         self.insert_actiongroup.set_sensitive(not preview)
@@ -405,11 +438,11 @@ class MainWindow:
 
     def on_preview_button_clicked(self, button):
         self.journal.save_old_day()
-        if browser.WebKit2:
+        if self.html_editor.internal:
             self.html_editor.show_day(self.day)
             self.change_mode(preview=True)
         else:
-            date_format = self.journal.config.read('exportDateFormat', '%A, %x')
+            date_format = self.journal.config.read('exportDateFormat')
             date_string = dates.format_date(date_format, self.day.date)
             markup_string = markup.get_markup_for_day(self.day)
             html = self.journal.convert(
@@ -502,13 +535,22 @@ class MainWindow:
             if action.is_user_gesture():
                 uri = action.get_request().get_uri()
                 logging.info('Clicked URI "%s"' % uri)
-                filesystem.open_url(uri)
+
+                if urls.is_entry_reference_uri(uri):
+                    self.navigate_to_referenced_entry(uri)
+                else:
+                    urls.open_url(uri)
 
                 decision.ignore()
 
         # Stop processing this event.
         return True
 
+    def navigate_to_referenced_entry(self, entry_reference_uri):
+        entry_reference_uri = urllib.parse.urlparse(entry_reference_uri)
+        date = dates.get_date_from_date_string(entry_reference_uri.fragment)
+        self.journal.change_date(date)
+
     def get_new_journal_dir(self, title, message):
         dir_chooser = self.builder.get_object('dir_chooser')
         dir_chooser.set_transient_for(self.main_frame)
@@ -637,7 +679,7 @@ class MainWindow:
         self.day_text_field.show_day(day)
 
         # Only switch mode automatically if set in preferences.
-        if self.journal.config.read('autoSwitchMode') and browser.WebKit2:
+        if self.journal.config.read('autoSwitchMode') and self.html_editor.internal:
             if day.has_text and not self.preview_mode:
                 self.change_mode(preview=True)
             elif not day.has_text and self.preview_mode:
@@ -648,7 +690,6 @@ class MainWindow:
             self.html_editor.show_day(day)
 
         self.categories_tree_view.set_day_content(day)
-        self.undo_redo_manager.set_stack(new_date)
 
     def get_day_text(self):
         return self.day_text_field.get_text()
@@ -663,48 +704,107 @@ class MainWindow:
         else:
             self.statusbar.show_message(title, msg, msg_type)
 
+    def disable_undo_redo_buttons(self):
+        self.undo_action.set_sensitive(False)
+        self.redo_action.set_sensitive(False)
+
+    def update_undo_redo_buttons(self, _gobject=None):
+        """Enable/disable undo+redo actions according to the current text buffer
+
+        The _gobject parameter is unused, but it's necessary for the method
+        to be connected to a GObject signal.
+        """
+        can_undo = self.day_text_field.day_text_buffer.can_undo()
+        self.undo_action.set_sensitive(can_undo)
+        can_redo = self.day_text_field.day_text_buffer.can_redo()
+        self.redo_action.set_sensitive(can_redo)
+
 
 class DayEditor(editor.Editor):
+    n_recent_buffers = 10  # How many recent buffers to store
+    _t2t_highlighting = None
+    _style_scheme = None
+
     def __init__(self, *args, **kwargs):
         editor.Editor.__init__(self, *args, **kwargs)
         self.day = None
-        self.scrolled_win = self.day_text_view.get_parent()
+        # Store buffers for recently edited days - these preserve undo history
+        # and cursor position. Once a buffer drops out of this, it needs to be
+        # recreated: at this point, the cursor and undo are lost.
+        self.recent_buffers = OrderedDict()
+
+    def _get_t2t_highlighting(self):
+        if self._t2t_highlighting is None:
+            # Load our own copy of t2t syntax highlighting
+            lm = GtkSource.LanguageManager.get_default()
+            search_path = lm.get_search_path()
+            if filesystem.files_dir not in search_path:
+                search_path.insert(0, filesystem.files_dir)
+                lm.set_search_path(search_path)
+            self._t2t_highlighting = lm.get_language('t2t')
+        return self._t2t_highlighting
+
+    def _get_style_scheme(self):
+        if self._style_scheme is None:
+            # Load our customised variant of the Tango scheme
+            sm = GtkSource.StyleSchemeManager.get_default()
+            if filesystem.files_dir not in sm.get_search_path():
+                sm.prepend_search_path(filesystem.files_dir)
+            self._style_scheme = sm.get_scheme('rednotebook')
+        return self._style_scheme
+
+    def _get_buffer(self, key, text):
+        '''Get an editing buffer for a given item
+
+        If key is in our cache of recently used buffers, its buffer is retrieved
+        and text is ignored. Otherwise, a new buffer is constructed with text.
+        '''
+        if key in self.recent_buffers:
+            self.recent_buffers.move_to_end(key)
+            return self.recent_buffers[key]
+
+        buf = self.recent_buffers[key] = GtkSource.Buffer.new()
+        buf.set_style_scheme(self._get_style_scheme())
+        buf.set_language(self._get_t2t_highlighting())
+        buf.create_tag('highlighter', background='Yellow')
+        buf.begin_not_undoable_action()
+        buf.set_text(text)
+        buf.end_not_undoable_action()
+
+        if len(self.recent_buffers) > self.n_recent_buffers:
+            self.recent_buffers.popitem(last=False)
+
+        # Only one buffer is added at a time, so the 'if' above should always
+        # keep us at most n_recent_buffers. If code is added to add to the list
+        # elsewhere, it should check the maximum length as well.
+        assert len(self.recent_buffers) <= self.n_recent_buffers
+
+        return buf
+
+    def _get_buffer_for_day(self, day):
+        return self._get_buffer(day.date, day.text)
 
     def show_day(self, new_day):
-        # Save the position in the edit pane for the old day
-        if self.day:
-            cursor_pos = self.day_text_buffer.get_property('cursor-position')
-            # If there is a selection we save it, else we save the cursor position
-            selection = self.day_text_buffer.get_selection_bounds()
-            if selection:
-                selection = [it.get_offset() for it in selection]
-            else:
-                selection = [cursor_pos, cursor_pos]
-            self.day.last_edit_pos = (self.scrolled_win.get_hscrollbar().get_value(),
-                                      self.scrolled_win.get_vscrollbar().get_value(),
-                                      selection)
-
         # Show new day
         self.day = new_day
-        self.set_text(self.day.text, undoing=True)
+        buf = self._get_buffer_for_day(new_day)
+        self.replace_buffer(buf)
+        self.day_text_view.grab_focus()
 
         if self.search_text:
             # If a search is currently made, scroll to the text and return.
             GObject.idle_add(self.scroll_to_text, self.search_text)
+            GObject.idle_add(self.highlight, self.search_text)
             return
 
-        if self.day.last_edit_pos is not None:
-            x, y, selection = self.day.last_edit_pos
-            GObject.idle_add(self.scrolled_win.get_hscrollbar().set_value, x)
-            GObject.idle_add(self.scrolled_win.get_vscrollbar().set_value, y)
-            GObject.idle_add(self._restore_selection, selection)
-
-    def _restore_selection(self, selection):
-        iters = [self.day_text_buffer.get_iter_at_offset(offset)
-                 for offset in selection]
-        self.day_text_buffer.select_range(*iters)
+    def show_template(self, title, text):
+        buf = self._get_buffer(('template', title), text)
+        self.replace_buffer(buf)
         self.day_text_view.grab_focus()
 
+    def clear_buffers(self):
+        self.recent_buffers.clear()
+
 
 class NewEntryDialog:
     def __init__(self, main_frame):
@@ -795,7 +895,7 @@ class Statusbar:
 
     def show_message(self, title, msg, msg_type):
         if title and msg:
-            text = '%s: %s' % (title, msg)
+            text = '{}: {}'.format(title, msg)
         else:
             text = title or msg
         self._show_text(text)
