@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009  Jendrik Seipp
 #
@@ -41,6 +40,7 @@ except ImportError as err:
     sys.exit(1)
 
 gi.require_version("Gtk", "3.0")
+gi.require_version('GtkSource', '3.0')
 
 if hasattr(sys, "frozen"):
     base_dir = sys._MEIPASS
@@ -92,6 +92,7 @@ elibintl.install(GETTEXT_DOMAIN, LOCALE_PATH, libintl=None)
 
 from rednotebook.util import utils
 from rednotebook.util import markup
+from rednotebook.help import example_content
 from rednotebook import info
 from rednotebook import configuration
 from rednotebook import data
@@ -153,7 +154,7 @@ try:
     from gi.repository import GObject
 except (ImportError, AssertionError) as e:
     logging.error(e)
-    logging.error('GTK+ not found. Please install it (gir1.2-gtk-3.0).')
+    logging.error('GTK not found. Please install it (gir1.2-gtk-3.0).')
     sys.exit(1)
 
 GObject.threads_init()
@@ -164,7 +165,6 @@ from rednotebook import backup
 
 from rednotebook.util.statistics import Statistics
 from rednotebook.gui.main_window import MainWindow
-from rednotebook import index
 from rednotebook import storage
 from rednotebook.data import Month
 
@@ -187,8 +187,6 @@ class Journal:
         self.date = None
         self.months = {}
 
-        self.search_index = index.Index()
-
         # The dir name is the title
         self.title = ''
 
@@ -297,7 +295,7 @@ class Journal:
 
         try:
             filesystem.make_directory(self.dirs.data_dir)
-        except (OSError, IOError) as err:
+        except OSError as err:
             logging.error('Creating journal directory failed: {}'.format(err))
             self.frame.show_save_error_dialog(exit_imminent)
             return True
@@ -305,7 +303,7 @@ class Journal:
         try:
             something_saved = storage.save_months_to_disk(
                 self.months, self.dirs.data_dir, exit_imminent, saveas)
-        except (IOError, OSError) as err:
+        except OSError as err:
             logging.error('Saving month files failed: {}'.format(err))
             self.frame.show_save_error_dialog(exit_imminent)
             something_saved = None
@@ -322,8 +320,10 @@ class Journal:
         self.config.save_to_disk()
 
         if not (exit_imminent or changing_journal) and something_saved:
-            # Update cloud
+            # Update cloud.
             self.frame.cloud.update(force_update=True)
+            for tag in self.get_escaped_tags():
+                self.frame.search_box.add_entry(tag)
 
         # tell gobject to keep saving the content in regular intervals
         return True
@@ -343,7 +343,7 @@ class Journal:
         self.month = None
         self.months.clear()
         self.frame.search_box.clear()
-        self.search_index.clear()
+        self.frame.day_text_field.clear_buffers()
 
         self.months = storage.load_all_months_from_disk(data_dir)
 
@@ -353,19 +353,12 @@ class Journal:
         if self.is_first_start and not os.listdir(data_dir) and not self.days:
             self.add_instruction_content()
 
-        # We can't use self.days here since it uses self.save_old_day.
-        for month in self.months.values():
-            for day in month.days.values():
-                self.search_index.add(day.date, day.get_indexed_words())
-
         self.stats = Statistics(self)
 
         self.frame.cloud.update(force_update=True)
 
         self.frame.categories_tree_view.categories = self.categories
-        # Add auto-completion for tag search
-        self.frame.search_box.set_entries(
-            ['#%s' % data.escape_tag(tag) for tag in self.categories])
+        self.frame.search_box.set_entries(self.get_escaped_tags())
 
         self.title = filesystem.get_journal_title(data_dir)
 
@@ -383,7 +376,7 @@ class Journal:
         parts = ['RedNotebook']
         if self.title != 'data':
             parts.append(self.title)
-        parts.append(dates.format_date('%x', self.date))
+        parts.append(dates.format_date(self.config.read('exportDateFormat'), self.date))
         self.frame.main_frame.set_title(' - '.join(parts))
 
     def get_month(self, date):
@@ -403,14 +396,15 @@ class Journal:
     def get_day(self, date):
         return self.get_month(date).get_day(date.day)
 
+    def get_escaped_tags(self):
+        return ['#%s' % data.escape_tag(tag) for tag in self.categories]
+
     def save_old_day(self):
         '''Order is important'''
-        self.search_index.remove(self.day.date, self.day.get_indexed_words())
         old_content = self.day.content
         new_content = self.frame.categories_tree_view.get_day_content()
         new_content['text'] = self.frame.get_day_text()
         self.day.content = new_content
-        self.search_index.add(self.day.date, self.day.get_indexed_words())
 
         content_changed = (old_content != new_content)
         if content_changed:
@@ -429,18 +423,6 @@ class Journal:
 
         self.set_frame_title()
 
-    def merge_days(self, days):
-        '''
-        Method used by importers
-        '''
-        self.save_old_day()
-        for new_day in days:
-            date = new_day.date
-            month = self.get_month(date)
-            old_day = month.get_day(date.day)
-            old_day.merge(new_day)
-            month.edited = True
-
     @property
     def day(self):
         return self.month.get_day(self.date.day)
@@ -483,12 +465,12 @@ class Journal:
             log_level = logging.INFO
 
         self.frame.show_message(title, msg, msg_type)
-        logging.log(log_level, '%s. %s' % (title, msg) if title else msg)
+        logging.log(log_level, '{}. {}'.format(title, msg) if title else msg)
 
     @property
     def categories(self):
-        return list(sorted(set(itertools.chain.from_iterable(
-            day.categories for day in self.days)), key=locale.strxfrm))
+        return sorted(set(itertools.chain.from_iterable(
+            day.categories for day in self.days)), key=locale.strxfrm)
 
     def get_entries(self, category):
         entries = set()
@@ -498,26 +480,8 @@ class Journal:
 
     def search(self, text, tags):
         results = []
-        # TODO: Allow using index with a configuration option?
-        use_index = False
-        if use_index:
-            words = data.get_indexed_words(text)
-            words.extend('#{}'.format(tag) for tag in tags)
-
-            if not words:
-                return []
-
-            dates = self.search_index.find(words[0])
-            for word in words[1:]:
-                dates &= self.search_index.find(word)
-
-            for date in sorted(dates, reverse=True):
-                for word in words:
-                    results.append(self.get_day(date).search(word, tags))
-        else:
-            days = self.get_days_with_tags(tags)
-            for day in reversed(days):
-                results.append(day.search(text, tags))
+        for day in reversed(self.get_days_with_tags(tags)):
+            results.append(day.search(text, tags))
         return results
 
     def get_days_with_tags(self, tags):
@@ -525,7 +489,7 @@ class Journal:
             return self.days
         days = []
         for day in self.days:
-            day_tags = set(data.escape_tag(tag) for tag in day.categories)
+            day_tags = {data.escape_tag(tag) for tag in day.categories}
             if all(tag in day_tags for tag in tags):
                 days.append(day)
         return days
@@ -585,10 +549,9 @@ class Journal:
 
         logging.info('Adding example content on %s' % current_date)
 
-        for example_day in info.example_content:
+        for example_day in example_content:
             self.day.content = example_day
             self.frame.set_date(self.month, self.date, self.day)
-            self.search_index.add(self.day.date, self.day.get_indexed_words())
             self.go_to_next_day()
 
         self.change_date(current_date)
