@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009  Jendrik Seipp
 #
@@ -28,6 +27,7 @@ from gi.repository import Pango
 from rednotebook.external import txt2tags
 from rednotebook.data import HASHTAG
 from rednotebook.util import filesystem
+from rednotebook.util import urls
 
 
 # Linebreaks are only allowed at line ends
@@ -37,7 +37,7 @@ REGEX_HTML_LINK = r'<a.*?>(.*?)</a>'
 # pic [""/home/user/Desktop/RedNotebook pic"".png]
 PIC_NAME = r'\S.*?\S|\S'
 PIC_EXT = r'(?:png|jpe?g|gif|eps|bmp|svg)'
-REGEX_PIC = re.compile(r'(\["")(%s)("")(\.%s)(\?\d+)?(\])' % (PIC_NAME, PIC_EXT), flags=re.I)
+REGEX_PIC = re.compile(r'(\["")({})("")(\.{})(\?\d+)?(\])'.format(PIC_NAME, PIC_EXT), flags=re.I)
 
 # named local link [my file.txt ""file:///home/user/my file.txt""]
 # named link in web [heise ""http://heise.de""]
@@ -49,7 +49,6 @@ COLOR_ESCAPED = r'XBEGINCOLORX(.*?)XSEPARATORX(.*?)XENDCOLORX'
 TABLE_HEAD_BG = '#aaa'
 
 CHARSET_UTF8 = '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />'
-PRINT_FUNCTION = '<script></script>'
 
 CSS = """\
 <style type="text/css">
@@ -137,7 +136,7 @@ def convert_categories_to_markup(categories, with_category_title=True):
     return markup
 
 
-def get_markup_for_day(day, with_text=True, with_tags=True, categories=None, date=None):
+def get_markup_for_day(day, target, with_text=True, with_tags=True, categories=None, date=None):
     '''
     Used for exporting days
     '''
@@ -145,7 +144,12 @@ def get_markup_for_day(day, with_text=True, with_tags=True, categories=None, dat
 
     # Add date if it is not None and not the empty string
     if date:
-        export_string += '= %s =\n\n' % date
+        if target in ['xhtml', 'html']:
+            # Following anchor will be used as a target for every entry reference mentioning
+            # this entry's date.
+            export_string += '\'\'<span id="{:%Y-%m-%d}"></span>\'\'\n'.format(day.date)
+
+        export_string += '= {} =\n\n'.format(date)
 
     # Add text
     if with_text:
@@ -156,8 +160,9 @@ def get_markup_for_day(day, with_text=True, with_tags=True, categories=None, dat
 
     if with_tags and categories:
         categories = [word.lower() for word in categories]
-        export_categories = dict((x, y) for (x, y) in category_content_pairs.items()
-                                 if x.lower() in categories)
+        export_categories = {
+            x: y for (x, y) in category_content_pairs.items()
+            if x.lower() in categories}
     elif with_tags and categories is None:
         # No restrictions
         export_categories = category_content_pairs
@@ -181,9 +186,6 @@ def get_markup_for_day(day, with_text=True, with_tags=True, categories=None, dat
 
 
 def _get_config(target, options):
-
-    config = {}
-
     # Set the configuration on the 'config' dict.
     config = txt2tags.ConfigMaster()._get_defaults()
 
@@ -225,6 +227,15 @@ def _get_config(target, options):
         # {{red text|color:red}} -> <span style="color:red">red text</span>
         config['postproc'].append([COLOR_ESCAPED, r'<span style="color:\2">\1</span>'])
 
+        # Custom css
+        fonts = options.pop('font', 'sans-serif')
+        css = CSS % {'font': fonts, 'table_head_bg': TABLE_HEAD_BG}
+        config['postproc'].append([r'</head>', css + '</head>'])
+
+        # MathJax
+        if options.pop('add_mathjax'):
+            config['postproc'].append([r'</body>', MATHJAX + '</body>'])
+
     elif target == 'tex':
         config['encoding'] = 'utf8'
         config['preproc'].append(['€', 'Euro'])
@@ -258,11 +269,12 @@ def _get_config(target, options):
         config['preproc'].append([r'\\\[\s*(.+?)\s*\\\]', r"BEGINEQUATION''\1''ENDEQUATION"])
         config['preproc'].append([r'\$\$\s*(.+?)\s*\$\$', r"BEGINEQUATION''\1''ENDEQUATION"])
         config['postproc'].append([r'BEGINEQUATION(.+)ENDEQUATION', r'$$\1$$'])
+
         config['preproc'].append([r'\\\(\s*(.+?)\s*\\\)', r"BEGINMATH''\1''ENDMATH"])
         config['postproc'].append([r'BEGINMATH(.+)ENDMATH', r'$\1$'])
 
         # Fix utf8 quotations - „, “ and ” cause problems compiling the latex document.
-        config['postproc'].extend([[u'„', '"'], [u'”', '"'], [u'“', '"']])
+        config['postproc'].extend([['„', '"'], ['”', '"'], ['“', '"']])
 
         # Enable index.
         config['style'].append('makeidx')
@@ -279,29 +291,34 @@ def _get_config(target, options):
         # Apply image resizing ([WIDTH400-file:///pathtoimage.jpg])
         config['postproc'].append([r'\[WIDTH(\d+)-(.+)\]', r'[\2?\1]'])
 
+    # Entry references
+    if target in ['xhtml', 'html']:
+        # txt2tags will generate links to the named entry references because they share common bracket
+        # notation used by the URIs. We just need to add our internal schema to make it a proper URI.
+        config['preproc'].append([r'\[(?P<name>.+)\s+(?P<date>\d{4}-\d{2}-\d{2})\s*\]',
+                                  r'[\g<name> #\g<date>]'])
+
+        # Convert bracketed dates into named references where the date itself is being used as a name.
+        # For example:
+        # "Today is [2019-10-20]" will be converted into "Today is [2019-10-20 #2019-10-20]"
+        config['preproc'].append([r'\[(?P<date>\d{4}-\d{2}-\d{2})\]',
+                                  r'[\g<date> #\g<date>]'])
+    else:
+        # Links to entry references are not supported for targets other than (X)HTML
+        config['preproc'].append([r'\[(?P<name>.+)\s+(?P<date>\d{4}-\d{2}-\d{2})\]',
+                                  r'\g<name> (\g<date>)'])
+
     # Allow resizing images by changing
     # [filename.png?width] to [WIDTHwidth-filename.png]
     img_ext = r'png|jpe?g|gif|eps|bmp|svg'
     img_name = r'\S.*\S|\S'
 
     # Apply this prepoc only after the latex image quotes have been added
-    config['preproc'].append([r'\[(%s\.(%s))\?(\d+)\]' % (img_name, img_ext), r'[WIDTH\3-\1]'])
+    config['preproc'].append([r'\[({}\.({}))\?(\d+)\]'.format(img_name, img_ext), r'[WIDTH\3-\1]'])
 
     # Disable colors for all other targets.
     config['postproc'].append([COLOR_ESCAPED, r'\1'])
 
-    # MathJax
-    if options.pop('add_mathjax'):
-        config['postproc'].append([r'</body>', MATHJAX + '</body>'])
-
-    config['postproc'].append([r'</body>', PRINT_FUNCTION + '</body>'])
-
-    # Custom css
-    fonts = options.pop('font', 'sans-serif')
-    if 'html' in target:
-        css = CSS % {'font': fonts, 'table_head_bg': TABLE_HEAD_BG}
-        config['postproc'].append([r'</head>', css + '</head>'])
-
     config.update(options)
 
     return config
@@ -316,7 +333,7 @@ def _convert_paths(txt, data_dir):
             path = os.path.join(data_dir, path)
             assert os.path.isabs(path), path
             if os.path.exists(path):
-                uri = filesystem.get_local_url(path)
+                uri = urls.get_local_url(path)
         return uri
 
     def _convert_pic_path(match):
@@ -467,15 +484,15 @@ def convert_to_pango(txt, headers=None, options=None):
 
 def convert_from_pango(pango_markup):
     original_txt = pango_markup
-    replacements = dict((
-        ('<b>', '**'), ('</b>', '**'),
-        ('<i>', '//'), ('</i>', '//'),
-        ('<s>', '--'), ('</s>', '--'),
-        ('<u>', '__'), ('</u>', '__'),
-        ('&amp;', '&'),
-        ('&lt;', '<'), ('&gt;', '>'),
-        ('\n', r'\\'),
-    ))
+    replacements = {
+        '<b>': '**', '</b>': '**',
+        '<i>': '//', '</i>': '//',
+        '<s>': '--', '</s>': '--',
+        '<u>': '__', '</u>': '__',
+        '&amp;': '&',
+        '&lt;': '<', '&gt;': '>',
+        '\n': r'\\',
+    }
     for orig, repl in replacements.items():
         pango_markup = pango_markup.replace(orig, repl)
 
