@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009, 2014, 2015  Jendrik Seipp
 #
@@ -37,6 +36,8 @@ class Config(dict):
         'autoSwitchMode': 0,
         'closeToTray': 0,
         'checkForNewVersion': 0,
+        'dateTimeString': '%A, %x %X',
+        'exportDateFormat': '%A, %x',
         'showTagsPane': 0,
         'weekNumbers': 0,
         'portable': 0,
@@ -49,18 +50,19 @@ class Config(dict):
         'mainFrameMaximized': 0,
         'mainFrameX': None,
         'mainFrameY': None,
-        'leftDividerPosition': 250,
+        'leftDividerPosition': 260,
         'rightDividerPosition': None,
+        'cloudMaxTags': 1000,
     }
 
-    obsolete_keys = set([
+    obsolete_keys = {
         'useGTKMozembed', 'useWebkit', 'LD_LIBRARY_PATH',
         'MOZILLA_FIVE_HOME', 'cloudTabActive', 'mainFontSize',
         'running'
-    ])
+    }
 
     # Allow changing the value of portable only in default.cfg.
-    suppressed_keys = set(['portable', 'user_dir'])
+    suppressed_keys = {'portable', 'user_dir'}
 
     def __init__(self, config_file):
         dict.__init__(self)
@@ -142,12 +144,12 @@ class Config(dict):
         lines = []
         for key, value in sorted(self.items()):
             if key not in self.suppressed_keys:
-                lines.append('%s=%s' % (key, value))
+                lines.append('{}={}'.format(key, value))
 
         try:
             filesystem.make_directory(os.path.dirname(self.filename))
             filesystem.write_file(self.filename, '\n'.join(lines))
-        except IOError:
+        except OSError:
             logging.error('Configuration could not be saved. Please check '
                           'your permissions')
         else:
