@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009  Jendrik Seipp
 #
@@ -23,10 +22,10 @@ from gi.repository import GdkPixbuf
 from gi.repository import Gtk
 
 from rednotebook.util import utils
+from rednotebook.help import help_text
 from rednotebook import info
 from rednotebook.util import filesystem
 from rednotebook import storage
-# from rednotebook.gui.imports import ImportAssistant
 from rednotebook.gui import insert_menu
 from rednotebook.gui import format_menu
 
@@ -41,7 +40,6 @@ MENUBAR_XML = '''\
         <menuitem action="Save"/>
         <menuitem action="SaveAs"/>
         <separator/>
-        <!--<menuitem action="Import"/>-->
         <menuitem action="Export"/>
         <menuitem action="Backup"/>
         <menuitem action="Statistics"/>
@@ -104,9 +102,6 @@ class MainMenuBar:
                 None, _('Save journal at a new location. The old journal files will also be saved'),
                 self.on_save_as_menu_item_activate),
             # Translators: Verb
-            # ('Import', Gtk.STOCK_ADD, _('_Import'),
-            #    None, _('Open the import assistant'), self.on_import_menu_item_activate),
-            # Translators: Verb
             ('Export', Gtk.STOCK_CONVERT, _('Export'),
                 '<Ctrl>e', _('Open the export assistant'), self.on_export_menu_item_activate),
             # Translators: Verb
@@ -197,8 +192,8 @@ class MainMenuBar:
         self.journal.open_journal(new_dir)
 
     def on_new_journal_button_activate(self, widget):
-        msg = ('%s\n%s' % (_('Journals are saved in a directory, not in a single file.'),
-                           _('The directory name will be the title of the new journal.')))
+        msg = ('{}\n{}'.format(_('Journals are saved in a directory, not in a single file.'),
+                               _('The directory name will be the title of the new journal.')))
         self.select_journal('new', _('Select an empty folder for your new journal'), msg)
 
     def on_open_journal_button_activate(self, widget):
@@ -218,11 +213,11 @@ class MainMenuBar:
 
     def on_undo(self, widget):
         editor = self.main_window.day_text_field
-        editor.add_undo_point()
-        self.main_window.undo_redo_manager.undo()
+        editor.day_text_buffer.undo()
 
     def on_redo(self, widget):
-        self.main_window.undo_redo_manager.redo()
+        editor = self.main_window.day_text_field
+        editor.day_text_buffer.redo()
 
     def _get_active_editor_widget(self):
         if self.main_window.preview_mode:
@@ -260,10 +255,6 @@ class MainMenuBar:
     def on_backup_activate(self, widget):
         self.journal.archiver.backup()
 
-    # def on_import_menu_item_activate(self, widget):
-    #     assistant = ImportAssistant(self.journal)
-    #     assistant.run()
-
     def on_export_menu_item_activate(self, widget):
         self.journal.save_old_day()
         self.main_window.export_assistant.run()
@@ -273,9 +264,9 @@ class MainMenuBar:
 
     def on_help_menu_item_activate(self, widget):
         temp_dir = self.journal.dirs.temp_dir
-        filesystem.write_file(os.path.join(temp_dir, 'source.txt'), info.help_text)
+        filesystem.write_file(os.path.join(temp_dir, 'source.txt'), help_text)
         html = self.journal.convert(
-            info.help_text, 'xhtml',
+            help_text, 'xhtml',
             headers=[_('RedNotebook Documentation'), info.version, ''],
             options={'toc': 1})
         utils.show_html_in_browser(html, os.path.join(temp_dir, 'help.html'))
