@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009  Jendrik Seipp
 #
@@ -213,7 +212,7 @@ class TemplateManager:
         self.merge_id = None
         self.actiongroup = None
         self.tmp_title = None
-        self.tmp_parts = None
+        self.selection_to_replace = None
 
         self._template_mode_info_bar = TemplateInfo()
         self._template_mode_info_bar.hide()
@@ -243,20 +242,19 @@ class TemplateManager:
                 self.main_window.uimanager.get_widget('/MainMenuBar/Edit/Find')]:
             widget.set_sensitive(sensitive)
 
-    def enter_template_mode(self, title, parts):
+    def enter_template_mode(self, title, selection):
         # Save the templates title and the day's text.
         self.tmp_title = title
-        self.tmp_parts = parts
+        self.selection_to_replace = selection
         self.main_window.template_bar.show()
         text = self.get_text(title)
-        self.main_window.undo_redo_manager.set_stack(title)
-        self.main_window.day_text_field.set_text(text, undoing=True)
+        self.main_window.day_text_field.show_template(title, text)
         self._template_mode_info_bar.show()
         self._set_widgets_sensitive(False)
 
     def exit_template_mode(self):
         self.tmp_title = None
-        self.tmp_parts = None
+        self.selection_to_replace = None
         self.main_window.template_bar.hide()
         if self.main_window.preview_mode:
             self.main_window.change_mode(preview=False)
@@ -264,17 +262,14 @@ class TemplateManager:
         self._template_mode_info_bar.hide()
         self._set_widgets_sensitive(True)
 
-    def _reset_undo_stack(self):
-        self.main_window.undo_redo_manager.set_stack(self.main_window.day.date)
-
     def edit(self, title):
-        parts = self.main_window.day_text_field.get_text_parts()
-        self.enter_template_mode(title, parts)
+        selection = self.main_window.day_text_field.get_selection_bounds()
+        self.enter_template_mode(title, selection)
 
     def _replace_macros(self, text):
         # convert every "$date$" to the current date
         config = self.main_window.journal.config
-        format_string = config.read('dateTimeString', '%A, %x %X')
+        format_string = config.read('dateTimeString')
         date_string = dates.format_date(format_string)
         text = text.replace('$date$', date_string)
         return text
@@ -283,11 +278,14 @@ class TemplateManager:
         self.on_save(None)
         template = self.main_window.day_text_field.get_text()
         template = self._replace_macros(template)
-        p1, p2, p3 = self.tmp_parts
-        self._reset_undo_stack()
-        # Force addition of an undo item.
-        self.main_window.day_text_field.set_text(p1 + p2 + p3, undoing=True)
-        self.main_window.day_text_field.set_text(p1 + template + p3)
+
+        self.main_window.day_text_field.show_day(self.main_window.day)
+        buf = self.main_window.day_text_field.day_text_buffer
+        buf.begin_user_action()
+        buf.select_range(*self.selection_to_replace)
+        buf.delete_selection(interactive=False, default_editable=True)
+        buf.insert_at_cursor(template)
+        buf.end_user_action()
         self.exit_template_mode()
 
     def on_save(self, button):
@@ -295,8 +293,7 @@ class TemplateManager:
         filesystem.write_file(self.get_path(self.tmp_title), template)
 
     def on_close(self, button):
-        self._reset_undo_stack()
-        self.main_window.day_text_field.set_text(''.join(self.tmp_parts), undoing=True)
+        self.main_window.day_text_field.show_day(self.main_window.day)
         self.exit_template_mode()
 
     def on_new_template(self, action):
@@ -325,17 +322,15 @@ class TemplateManager:
 
         if response == Gtk.ResponseType.OK:
             title = entry.get_text()
-            parts = self.main_window.day_text_field.get_text_parts()
+            selection = self.main_window.day_text_field.get_selection_bounds()
             path = self.get_path(title)
             filesystem.make_file(path, example_text)
-            self.enter_template_mode(title, parts)
+            self.enter_template_mode(title, selection)
 
     def _get_weekday_number(self):
-        return self.main_window.journal.date.weekday() + 1
+        return self.main_window.journal.date.isoweekday()
 
     def get_path(self, title):
-        if title == 'Weekday':
-            title = str(self._get_weekday_number())
         return os.path.join(self.dirs.template_dir, title + '.txt')
 
     def get_text(self, title):
@@ -397,7 +392,7 @@ class TemplateManager:
 
         actions.append(('EditWeekday', Gtk.STOCK_HOME,
                         _("This Weekday's Template"), None, None,
-                        lambda button: self.edit('Weekday')))
+                        lambda button: self.edit(str(self._get_weekday_number()))))
 
         actions.append(('NewTemplate', Gtk.STOCK_NEW, _('Create New Template'),
                         None, None, self.on_new_template))
